/*--------------------------------------------------------------------------
**  ComItAlg
**
**  Purpose:   Solve for optimization problem in commitment case. The economy
**             evolves as
**
**             x1(t+1)       =   A * x1(t)    + Bu(t) + e(t+1)
**             E(t)x2(t+1)           x2(t)              0
**
**             where x1(t) ("backward looking") has n1 elements,
**             and x2(t) ("forward looking") has n2 elements. The initial
**             value of x1, x1(0), is given by history. Let n=n1+n2.
**             u(t) is an kx1 vector of decision variables, which follows
**             the decision rule.
**
**             Define x(t)=[x1(t),x2(t)]. The loss function of the policy
**             maker is
**
**             Sum{ (beta^t)*[x(t)'Q*x(t)+2x(t)'U*u(t)+u(t)'R*u(t)],t=0,1,... }
**
**
**
**
**  Usage:     dlibrary -a PsDtgSen;           /*making dll available*/
**            { M,C } = ComItAlg( A,B,Q,R,U,beta,n1,n2,cutoff );
**             dlibrary -d;                    /*unloading dll*/
**
**  Input:     A        nxn matrix, (n=n1+n2)
**             B        nxk matrix
**             Q        nxn matrix, symmetric
**             R        kxk matrix, symmetric
**             U        nxk matrix
**             beta     scalar, discount factor (eg 0.99)
**             n1       scalar, # of predetermined variables
**             n2       scalar, # of forward looking variables
**             cutoff   scalar, max modulus of backward looking variables, eg. 1.0001
**
**  Output:    M        nxn matrix, [x1(t+1),p2(t+1)] = M*[x1(t),p2(t)] + e(t+1)
**             C        (n2+k+n1)xn matrix, [x2(t),u(t),p1(t)] = C*[x1(t),p2(t)]
**
**
**
**
**  Paul Sderlind, revised 25 Aug 2000
----------------------------------------------------------------------------*/

proc (2) = ComItAlg( A,B,Q,R,U,beta,n1,n2,cutoff );

  local n,k,G,D,G11,G12,G13,G14,D11,D12,D13,D14,T,S,Qa,Z,lambda,logcon,
        Stt,Std,Sdt,Sdd,Ttt,Ttd,Tdt,Tdd,Zkt,Zkd,Zlt,Zld,
        oldtrap,Zkt_1,Stt_1,M,C,vv1,vv2,vv3,vv4;


  Q = (Q + Q')/2;             /*to make symmetric*/
  R = (R + R')/2;

  n = n1 + n2;
  k = rows(R);                /*No. of control variables*/

  G =  ( eye(n)      ~ zeros(n,k) ~  zeros(n,n)    )|
       ( zeros(n,n)  ~ zeros(n,k) ~  (beta*A')     )|
       ( zeros(k,n)  ~ zeros(k,k) ~  (-B')         );

  D =  (  A         ~  B          ~  zeros(n,n)    )|
       ( (-beta*Q)  ~ (-beta*U)   ~  eye(n)        )|
       (  U'        ~  R          ~  zeros(k,n)    );

/*
  { G11,G12,G13,G14 } = Partit14Ps( G,n1,n2,k+n1,n2 );  /*x1,x2,(u,p1),p2*/
  { D11,D12,D13,D14 } = Partit14Ps( D,n1,n2,k+n1,n2 );  /*u(t) first in p1(t)*/
*/

  G11 = G[.,1:n1];
  G12 = G[.,n1+1:n1+n2];
  G13 = G[.,n1+n2+1:n1+n2+k+n1];
  G14 = G[.,n1+n2+k+n1+1:n1+n2+k+n1+n2];
  D11 = D[.,1:n1];
  D12 = D[.,n1+1:n1+n2];
  D13 = D[.,n1+n2+1:n1+n2+k+n1];
  D14 = D[.,n1+n2+k+n1+1:n1+n2+k+n1+n2];


  G = G11~G14~G12~G13;                 /*x1,x2,(u,p1),p2 -> x1,p2,x2,(u,p1)*/
  D = D11~D14~D12~D13;

#ifDLLCALL
     /*Generalized real Schur decomposition, Lapack, allowing singular R*/
  { T,S,Qa,Z,lambda } = DtgSen( D,G,cutoff,1,0 );  /*G=Q*S*ZH, D=Q*T*ZH*/
  logcon = packr(abs(lambda)) .<= cutoff;          /*Tii/Sii<cutoff first*/
#endif


  if sumc(logcon) < n;
    "\lToo few stable roots: no stable solution";
    retp( error(0),error(0) );                     /*returning missing values*/
  elseif sumc(logcon) > n;
    "\lToo many stable roots: inifite number of stable solutions";
    retp( error(0),error(0) );
  endif;


  Stt = S[1:n,1:n];
  Ttt = T[1:n,1:n];
  Zkt = Z[1:n,1:n];
  Zlt = Z[n+1:n+k+n,1:n];



  oldtrap = trapchk(1);     /*get ones bit*/
  trap 1,1;                 /*set ones bit only*/
  logcon = cond(Zkt) > 1e+14;
  trap oldtrap,1;           /*reset trap flag to previous value*/
  if  (logcon);
    "\lZkt is singular: rank condition for solution not satisfied";
    retp( error(0),error(0) );
  endif;

  Zkt_1 = inv(Zkt);
  Stt_1 = inv(Stt);


  M = real(Zkt*Stt_1*Ttt*Zkt_1);      /*[x1(t+1),p2(t+1)] = M*[x1(t),p2(t)]+e(t+1)*/
  C = real(Zlt*Zkt_1);                /*[x2(t),u(t),p1(t)] =C*[x1(t),p2(t)]*/
                                      /*getting rid of any trivial complex parts*/
  retp( M,C );

endp;
/*-----------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
**  DiscAlg
**
**  Purpose:    Solving the LQ problem under discretion, iterating
**              backwards in time.
**
**
**  Usage:      { M,C,V,F } = DiscAlg( A,B,Q,R,U,beta,n1,n2,Vt1,Ct1,
**                                     ConvCrit,Vweight,Fweight,CritLags,Step,
**                                     PrintIt,MaxIter );
**
**  Input:     A,B,Q,R,U,beta,n1,n2:   - see ComItAlg
**             Vt1        n1xn1 matrix: initial guess of value matrix
**             Ct1        n2xn1 matrix, initial guess of C in x2(t)=C*x1(t)
**             ConvCrit   2x1 convergence criteria for abs(V-Vt1)|abs(F-Ft1)
**             Vweight    scalar or n1xn1 matrix with weights for
**                        difference criterion for V
**             Fweight    scalar or (n1+n2)x1 vector with weights for
**                        difference criterion for F
**             CritLags   #lags of CritVar compared with ConvCrit
**             Step       scalar in (0,1): factor of updating of V and F as
**                        in Vt1 = step*V + (1-step)*Vt1
**             PrintIt    1: printing iteration number
**                        2: printing  iteration number and convergence criteria
**             MaxIter    scalar, maximum number of iterations (eg. 10000). NEW (March 2003)
**
**  Output:    M        n1x1 matrix, x1(t+1) = M*x1(t) + e(t+1)
**             C        n2xn1 matrix, x2(t)=C*x1(t)
**
**
**  Paul Sderlind
-----------------------------------------------------------------------*/

Proc (4) = DiscAlg( A,B,Q,R,U,beta,n1,n2,Vt1,Ct1,
                    ConvCrit,Vweight,Fweight,CritLags,Step,
                    PrintIt,MaxIter );

  local n,Ft1,Cdiff,iter,M,C,F,V,Vdiff,Fdiff,MaxDiff;


  Q = (Q + Q')/2;             /*to make symmetric*/
  R = (R + R')/2;

  n = n1 + n2;
  Ft1 = 1000;

  Cdiff = 1000*ones(1+CritLags,2);
  iter = 1;
  do until (maxc(Cdiff) < ConvCrit) or (iter >= MaxIter);          /*iterations*/

    { M,C,F,V } = DiscAlg2( A,B,Q,R,U,beta,n1,n2,Ct1,Vt1 ); /*solve period t*/

    Vdiff = maxc(  maxc( Vweight.*abs(V-Vt1) )  );        /*changes t+1 -> t*/
    Fdiff = maxc(  maxc( Fweight.*abs(F-Ft1) )  );
    Cdiff = trimr( Cdiff|(Vdiff~Fdiff),1,0 );             /*latest is last*/

    Vt1 = step*V + (1-step)*Vt1;                    /*"downdating"*/
    Ct1 = C;
    Ft1 = step*F + (1-step)*Ft1;

    if PrintIt == 1;
      iter;;
    elseif PrintIt == 2;
      iter~(maxc(Cdiff)');
    endif;

    iter = iter + 1;

  endo;                                 /*end iterations*/

  if iter >= MaxIter;
    "\lDiscAlg: Maximum number of iterations reached";
  endif;


  retp( M,C,V,F );

endp;

/*-----------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
**  DiscAlg2
**
**  Purpose:    Solving the LQ problem under discretion in t, given
**              C(t+1) and V(t+1).
**
**
**  Usage:     { M,C,Fs,Vs } = DiscAlg2( A,B,Q,R,U,beta,n1,n2,Ct1,Vst1 );
**
**  Input:     A,B,Q,R,U,beta,n1,n2: see ComItAlg
**             Ct1       -    n2xn1  matrix, C(t+1) in x2(t+1) = C(t+1)x1(t+1)
**             Vst1      -    n1xn1  matrix, V(t+1) value function matrix in t+1
**
**  Output:    M              n1xn1 matrix,
**             C              n2xn1 matrix,
**             Fs             kxn1 matrix,
**             Vs             n1xn1 matrix,
**
**
**
**
**
**  Paul Sderlind
-----------------------------------------------------------------------*/

proc (4) = DiscAlg2( A,B,Q,R,U,beta,n1,n2,Ct1,Vst1 );

  local n,A11,A12,A21,A22,Q11,Q12,Q21,Q22,B1,B2,U1,U2,
        d1mat,D,G,As,Bs,Qs,Us,Rs,Fs,Vs,C,M;

  n = n1 + n2;


  A11 = A[1:n1,1:n1];
  A12 = A[1:n1,(n1+1):n];
  A21 = A[(n1+1):n,1:n1];
  A22 = A[(n1+1):n,(n1+1):n];
  Q11 = Q[1:n1,1:n1];
  Q12 = Q[1:n1,(n1+1):n];
  Q21 = Q[(n1+1):n,1:n1];
  Q22 = Q[(n1+1):n,(n1+1):n];

  B1  = B[1:n1,.];
  B2  = B[n1+1:n,.];
  U1  = U[1:n1,.];
  U2  = U[n1+1:n,.];


  d1mat = inv(A22-Ct1*A12);
  D = d1mat * (Ct1*A11-A21);
  G = d1mat * (Ct1*B1-B2);

  As = A11 + A12*D;
  Bs = B1 + A12*G;

  Qs = Q11 + Q12*D + D'Q21 + D'Q22*D;
  Us = Q12*G + D'Q22*G + U1 + D'U2;
  Rs = R + G'Q22*G + G'U2 + U2'G;

  Fs = inv(Rs + beta*Bs'Vst1*Bs) * (Us' + beta*Bs'Vst1*As); /*u(t) = -F*x1(t)*/
  Vs = Qs - Us*Fs - Fs'Us' + Fs'Rs*Fs + beta*(As-Bs*Fs)'Vst1*(As-Bs*Fs);

  C = D - G*Fs;                   /*x2(t)=C*x1(t)*/
  M = A11 + A12*C - B1*Fs;        /*x1(t+1) = M*x1(t) + e(t+1)*/

  retp( M,C,Fs,Vs );

endp;
/*-----------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
**
**  SimpRulT
**
**  Purpose:   Solving RE model for given simple rule of the form
**             u(t)  = -F * x(t),   where   x(t)  = [x1(t)' x2(t)']'.
**
**
**
**
**  The shocks have the covariance matrix Cov[e(t)] = SigmaXX.
**
**
**
**
**
**  Usage:    dlibrary -a PsdGees;                /*making dll available*/
**            { M,C,J0 } = SimpRulT(A,B,Q,R,U,beta,n1,n2,F,SigmaXX,x10,cutoff);
**            dlibrary -d;                        /*unloading dll*/
**
**
**
**  Input:     A,B,Q,R,U,beta,n1,n2: see ComItAlg
**             F   -      kxn vector of decision rule
**             SigmaXX    n1xn1 covariance matrix of shocks
**             X_0        n1x1 vector, initial values for predetermined
**                        variables
**             cutoff     scalar, max modulus of backward looking variables
**
**
**  Output:    M      -   n1xn1 matrix, x1(t+1) = M*x1(t) + e(t+1)
**             C          n2xn1 matrix, x2(t) = C*x1(t)
**             J0         scalar, value of loss function
**
**
**  Paul Soderlind, 1999, revised Sep 2002
-----------------------------------------------------------------------*/

proc (3) = SimpRulT( A,B,Q,R,U,beta,n1,n2,F,SigmaXX,x10,cutoff );

  local n,T,Z,lambda,logcon,Zkt,Zkd,Zlt,Zld,Ttt,Ttd,Tdt,Tdd,oldtrap,Zkt_1,
        C,M,P,QUUR,PQUURP,V,J0,vecV;


  Q = (Q + Q')/2;             /*to make symmetric*/
  R = (R + R')/2;

  A = A - B*F;                     /*subst for u(t) in evolution*/
  n = n1 + n2;

#ifDLLCALL
  { T,Z,lambda } = Dgees( A,CutOff,0 );  /*Real Schur decomp, Lapack*/
  logcon = packr(abs(lambda)) .<= cutoff;
#endif


  if sumc(logcon) < n1;
    "\lToo few stable roots: no stable solution";
    retp( error(0),error(0),error(0) );                     /*returning missing values*/
  elseif sumc(logcon) > n1;
    "\lToo many stable roots: inifite number of stable solutions";
    retp( error(0),error(0),error(0) );
  endif;


  Zkt = Z[1:n1,1:n1];
  Zlt = Z[n1+1:n,1:n1];
  Ttt = T[1:n1,1:n1];


  oldtrap = trapchk(1);     /*get ones bit*/
  trap 1,1;                 /*set ones bit only*/
  logcon = cond(Zkt) > 1e+14;
  trap oldtrap,1;           /*reset trap flag to previous value*/
  if  (logcon);
    "\lZkt is singular: rank condition for solution not satisfied";
    retp( error(0),error(0),error(0) );   /*returning missing values*/
  endif;

  Zkt_1 = inv(Zkt);


  C = Zlt*Zkt_1;           /*x2(t) = C*x1(t)*/
  M = Zkt*Ttt*Zkt_1;       /*x1(t+1) = M*x1(t) + e(t+1)*/

  P = eye(n1)|C|(-F*(eye(n1)|C));
  QUUR = (Q~U)|(U'~R);
  PQUURP = P'QUUR*P;

  vecV = vec(PQUURP)/(eye(n1*n1) - (M').*.(beta*M'));
  V    = reshape(vecV,n1,n1)';                 /*solves V = W + beta*M'*V*M*/

  J0   =  x10'V*x10 + (beta/(1-beta)) * sumc(diag(V*SigmaXX));


  retp( M,C,J0 );

endp;

/*  OLD STUFF
  local vb,i,Vs1,Vdiff;
  i = 1;
  Vs1 = zeros(n1,n1);        /*iteraing on value matrix*/
  Vdiff = 100;
  do until vdiff < 1E-6;
    Vb = PQUURP + beta*M'Vs1*M;
    Vdiff = maxc(  maxc( abs(Vs1-Vb) )  );
    Vs1 = Vb;
    i = i + 1;
  endo;
  "testing V ";(V~(V-Vb));
*/
/*----------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
**  ComAlgS
**
**  Purpose:   Solve for optimization problem in STABLE commitment case.
**             The economy evolves as
**
**             x1(t+1)       =   A * x1(t)    + Bu(t) + e(t+1)
**
**             where x1(t) ("backward looking") has n1 elements. The initial
**             value of x1, x1(0), is given by history.
**             u(t) is an kx1 vector of decision variables, which follows
**             the decision rule.
**
**             The loss function of the policy
**             maker is
**
**             Sum{ (beta^t)*[x1(t)'Q*x1(t)+2x1(t)'U*u(t)+u(t)'Ru(t)],t=0,1,... }
**
**
**
**
**  Usage:     { M,C } = ComAlgS( A,B,Q,R,U,beta,cutoff );
**
**  Input:     A        n1xn1 matrix
**             B        n1xk matrix
**             Q        n1xn1 matrix
**             R        kxk matrix
**             U        n1xk matrix
**             beta     scalar, discount factor (eg 0.99)
**             cutoff   scalar, max modulus of stable roots, eg. 1.0001
**
**  Output:    M        n1xn1 matrix, x1(t+1) = M*x1(t) + e(t+1)
**             C        (k+n1)xn1 matrix, [u(t),p1(t)] = C*x1(t)
**
**
**
**
**  Paul Sderlind, July 2001
----------------------------------------------------------------------------*/

proc (2) = ComAlgS( A,B,Q,R,U,beta,cutoff);

  local n1,k,G,D,T,S,Qa,Z,lambda,logcon,Stt,Ttt,Zkt,Zlt, oldtrap,Zkt_1,Stt_1,
        M,C,maxEig;


  Q = (Q + Q')/2;             /*to make symmetric*/
  R = (R + R')/2;

  n1 = rows(A);               /*no of variables (all stable)*/
  k  = rows(R);               /*No. of control variables*/

  G =  ( eye(n1)       ~ zeros(n1,k) ~  zeros(n1,n1)  )|
       ( zeros(n1,n1)  ~ zeros(n1,k) ~  (beta*A')     )|
       ( zeros(k,n1)   ~ zeros(k,k)  ~  (-B')         );

  D =  (  A         ~  B          ~  zeros(n1,n1)   )|
       ( (-beta*Q)  ~ (-beta*U)   ~  eye(n1)        )|
       (  U'        ~  R          ~  zeros(k,n1)    );


#ifDLLCALL
     /*Generalized real Schur decomposition, Lapack, allowing singular R*/
  { T,S,Qa,Z,lambda } = DtgSen( D,G,cutoff,1,0 );  /*G=Q*S*ZH, D=Q*T*ZH*/
#endif


  Stt = S[1:n1,1:n1];
  Ttt = T[1:n1,1:n1];
  Zkt = Z[1:n1,1:n1];
  Zlt = Z[n1+1:n1+k+n1,1:n1];


  oldtrap = trapchk(1);     /*get ones bit*/
  trap 1,1;                 /*set ones bit only*/
  logcon = cond(Zkt) > 1e+14;
  trap oldtrap,1;           /*reset trap flag to previous value*/
  if  (logcon);
    "\lZkt is singular: rank condition for solution not satisfied";
    retp( error(0),error(0) );
  endif;

  Zkt_1 = inv(Zkt);
  Stt_1 = inv(Stt);


  M = real(Zkt*Stt_1*Ttt*Zkt_1);      /*x1(t+1)      = M*x1(t) + e(t+1)*/
  C = real(Zlt*Zkt_1);                /*[u(t),p1(t)] = C*x1(t)*/
                                      /*getting rid of any trivial complex parts*/

  maxEig = maxc(abs(eig(M)));
  if maxEig > cutoff;
    "\lSome abs(eigenvalue) of M larger than cutoff";
  endif;


  retp( M,C );

endp;
/*-----------------------------------------------------------------------*/

