/*----------------------------------------------------------------------------
**  ComAlgR   Solves for robust optimization problem in commitment case.
**
**            The economy evolves according to
**
**            [x1(t+1);E(t)x2(t+1)] = A*[x1(t);x2(t)] + B*u(t) + C*e(t+1),
**
**            where x1(t) ("backward looking") has n1 elements,
**            and x2(t) ("forward looking") has n2 elements. The initial
**            value of x1, x1(0), is given by history. Let n=n1+n2.
**            u(t) is an kx1 vector of decision variables, which follows
**            the decision rule.
**
**            Define x(t)=[x1(t),x2(t)]. The loss function of the policy
**            maker is
**
**            Sum{ (bet^t)*[x(t)'Q*x(t)+2x(t)'U*u(t)+u(t)'R*u(t)],t=0,1,... }
**
**
**
**  Usage:    dlibrary -a PsDtgSen;           @loading dll@
**            {M,N,Ma,Na,Fv,J0,J0a} = ComAlgR(A,B,Q,R,U,bet,n1,n2,cutoff,C,theta);
**
**
**  Input:    A        nxn matrix, (n=n1+n2)
**            B        nxk matrix
**            Q        nxn matrix, symmetric
**            R        kxk matrix, symmetric
**            U        nxk matrix
**            bet      scalar, discount factor (eg 0.99)
**            n1       scalar, # of predetermined variables
**            n2       scalar, # of forward looking variables
**            cutoff   scalar, max modulus of stable roots, eg. 1.0001
**            C        nxn1 matrix. CC' is the VCV matrix of the errors;
**                     the last n2 rows are zeros.
**            theta    scalar, (inverse of the) degree of robustness
**
**  Output:   M        nxn matrix, [x1(t+1),p2(t+1)] = M*[x1(t),p2(t)] + C*e(t+1)
**                     in the worst case solution, where p are multipliers.
**            N        (n2+k+n1*2)x(n1*2) matrix, [x2;u;v;p1]= N*[x1;p2], worst case
**            Ma       nxn matrix, [x1(t+1),p2(t+1)] = Ma*[x1(t),p2(t)] + C*e(t+1)
**                     in the approximating solution
**            Na       (n2+k+n1*2)x(n1*2) matrix, [x2;u;v;p1]= N*[x1;p2], approximating case
**            Fv       n1x(n1*2) matrix, v = -Fv*(x1;p2)
**            J0       scalar, loss function value, worst case
**            J0a      scalar, loss function value, approximating model
**
**
**  Note:    N and Na differs only in that rows corrsponding to v are zero in Na.
**
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Jun 2001
------------------------------------------------------------------------------*/

proc (7) = ComAlgR( A,B,Q,R,U,bet,n1,n2,cutoff,C,theta );

local k,n,Btilde,Utilde,Rtilde,M,Nbig,Nbig1,Nbig2,Nbig3,P,Fu,Fv,Mw,Ma,Na,maxEig,
      Ptilde,W,V,vecV,J0,J0a;


Q = (Q + Q')/2;             /*to make symmetric*/
R = (R + R')/2;

k = rows(R);                  /*no of control variables*/
n = n1 + n2;

if not (C[n1+1:n,.] == 0);
  "Non-zero elements found in the last n2 rows of C";
  retp(error(0),error(0),error(0),error(0),error(0),error(0),error(0));
endif;


Btilde = B ~ C;
Utilde = U ~ zeros(n1+n2,n1);
Rtilde = ( R ~            zeros(k,n1) ) |
         ( zeros(n1,k) ~ -theta*eye(n1) );

                                   /*commitment, robust policy*/
{M,Nbig} = ComItAlg( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,cutoff );


Nbig1 = Nbig[1:n2,.];                         /*x2 = N1*[x1;p2]*/
Nbig2 = Nbig[n2+1:n2+k,.];                    /* u = N2*[x1;p2]*/

Nbig3 = Nbig[n2+k+1:n2+k+n1,.];               /* v = N3*[x1;p2]*/

P = ( eye(n1) ~ zeros(n1,n2) )|
      Nbig1;                               /*[x1;x2] = P*[x1;p2]*/
Fu  = -Nbig2*inv(P);                        /*u = -Fu*[x1;x2]*/
Fv  = -Nbig3*inv(P);                        /*v = -Fv*[x1;x2]*/

Mw = inv(P)*(A-B*Fu-C*Fv)*P;     /*alternative expression for M, should equal M*/
/*"\lM-Mw";(M-Mw);*/

Ma = inv(P)*(A-B*Fu)*P;  /*[x1(t+1);p2(t+1)] = Ma*[x1(t);p2(t)], approximate model*/
Na = Nbig;               /*[x2;u;v;p1] = Na*[x1;p2], approximate model*/
Na[n2+k+1:n2+k+n1,.] = zeros(n1,n1+n2);  /*no evil shocks in approximate model*/
/*"\lM-Ma";(M-Ma);*/

maxEig = maxc(abs(eig(Ma)));
if maxEig > cutoff;
  "\lSome abs(eigenvalue) of Ma > cutoff";
endif;


Ptilde = ( eye(n1+n2)| -Fu| -Fv ) * P;           /*value function, worst case*/
W      = Ptilde' * ((Q~Utilde)| (Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n*n) - (M').*.(bet*M'));
V      = reshape(vecV,n,n)';                 /*solves V = W + bet*M'*V*M*/
if bet == 1;
  J0 = sumc( diag(C'*V*C*eye(n1)) );
else;
  J0 = bet/(1-bet) * sumc( diag(C'*V*C*eye(n1)) );
endif;


Ptilde = ( eye(n1+n2)| -Fu| zeros(n1,n1+n2) ) * P;   /*value function, approx*/
W      = Ptilde' * ((Q~Utilde)| (Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n*n) - (Ma').*.(bet*Ma'));
V      = reshape(vecV,n,n)';                 /*solves V = W + bet*Ma'*V*Ma*/
if bet == 1;
  J0a = sumc( diag(C'*V*C*eye(n1)) );
else;
  J0a = bet/(1-bet) * sumc( diag(C'*V*C*eye(n1)) );
endif;


retp( M,Nbig,Ma,Na,Fv,J0,J0a );

endp;

/*  OLD STUFF
local Vb,Vs_1,Vdiff;
Vs_1   = 0;
Vb     = 0;
Vdiff  = 100;
do while Vdiff > 1E-6;    /*iterating on value matrix V*/
  Vb    = W + bet*M'*Vs_1*M;  /*J(0) = Sum[(bet^t)z(t)'Wz(t),t=0,...]*/
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;            /*Vs_1 is V(s-1), V is V(s)*/
endo;
"testing V and V-Vb";V;(V-Vb);


local Vb,Vs_1,Vdiff;
Vs_1   = 0;
Vb     = 0;
Vdiff  = 100;
do while Vdiff > 1E-6;
  Vb    = W + bet*Ma'*Vs_1*Ma;
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;
endo;
"testing V and V-Vb";V;(V-Vb);
*/
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**  ComAlgSR  Solves for robust optimization problem in commitment case, for a
**            backward-looking system (no forward-looking variables).
**
**            The economy evolves according to
**
**            x(t+1) = A * x(t) + Bu(t) + C*e(t+1),
**
**            where x(t) is nx1, u(t) is kx1 and e(t+1) is nx1
**
**            The loss function of the policy
**            maker is
**
**            Sum{ (bet^t)*[x(t)'Q*x(t)+2x(t)'U*u(t)+u(t)'R*u(t)],t=0,1,... }
**
**
**
**
**  Usage:    dlibrary -a PsDtgSen;           @loading dll@
**            {M,N,Ma,Na,Fv,J0,J0a} = ComAlgSR(A,B,Q,R,U,bet,cutoff,C,theta);
**
**
**  Input:    A        see ComAlgR
**            B
**            Q
**            R
**            U
**            bet
**            cutoff
**            C
**            theta
**
**  Output:   M        nxn matrix, x(t+1) = M*x(t) + C*e(t+1), worst case
**            N        (k+n*2)x(n) matrix, [u;v;p1]= N*x, worst case
**            Ma       nxn matrix, x(t+1) = Ma*x(t) + C*e(t+1), approximating model
**            Na       (k+n*2)x(n) matrix, [u;v;p1]= Na*x, approximating model
**            Fv       nxn matrix, v = -Fv*x
**            J0       loss function value, worst case
**            J0a      loss function value, approximating model
**
**
**  Note:    N and Na differs only in that rows corrsponding to v are zero in Na.
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Jun 2001
------------------------------------------------------------------------------*/

proc (7) = ComAlgSR( A,B,Q,R,U,bet,cutoff,C,theta );

local n1,k,Btilde,Utilde,Rtilde,M,N,Nbig2,Nbig3,Fu,Fv,Mw,Ma,Na,maxEig,
      Ptilde,W,vecV,V,J0,J0a;


Q = (Q + Q')/2;             /*to make symmetric*/
R = (R + R')/2;

n1 = rows(A);                /*no of variables (all stable)*/
k  = rows(R);                /*no. of control variables*/

Btilde = B ~ C;
Utilde = U ~ zeros(n1,n1);
Rtilde = ( R           ~ zeros(k,n1)    )|
         ( zeros(n1,k) ~ -theta*eye(n1) );

                            /*commitment, robust policy*/

{M,N} = ComAlgS( A,Btilde,Q,Rtilde,Utilde,bet,cutoff );

Nbig2 = N[1:k,.];        /*u = N2*x; note: the P matrix is eye(n1) since n2=0*/
Nbig3 = N[k+1:k+n1,.];   /*v = N3*x*/

Fu  = -Nbig2;                /*u = -Fu*x*/
Fv  = -Nbig3;                /*v = -Fv*x*/

Mw = A-B*Fu-C*Fv;     /*alternative expression for M, should equal M*/
/*%disp(' ');disp('M-Mw');disp([M-Mw]);*/


Ma = A-B*Fu;                     /*x(t+1) = Ma*x(t), approximating model*/
Na = N;                          /*[u;v;p] = Na*x, approximating model*/
Na[k+1:k+n1,.] = zeros(n1,n1);   /*no evil shocks in approximate model*/

maxEig = maxc(abs(eig(Ma)));
if maxEig > cutoff;
  "\lSome abs(eigenvalue) of Ma > 1";
endif;
/*disp(' ');disp('M-Ma');disp([M-Ma]);*/



Ptilde =  eye(n1)| -Fu| -Fv;             /*value function, worst case*/
W      = Ptilde' * ((Q~Utilde)|(Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n1*n1) - (M').*.(bet*M'));
V      = reshape(vecV,n1,n1)';                 /*solves V = W + bet*M'*V*M*/
if bet == 1;
  J0 = sumc( diag(C'*V*C*eye(n1)) );
else;
  J0 = bet/(1-bet) * sumc( diag(C'*V*C*eye(n1)) );
endif;



Ptilde = eye(n1)| -Fu| zeros(n1,n1);             /*value function, approx*/
W      = Ptilde' * ((Q~Utilde)|(Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n1*n1) - (Ma').*.(bet*Ma'));
V      = reshape(vecV,n1,n1)';                 /*solves V = W + bet*Ma'*V*Ma*/
if bet == 1;
  J0a = sumc( diag(C'*V*C*eye(n1)) );
else;
  J0a = bet/(1-bet) * sumc( diag(C'*V*C*eye(n1)) );
endif;


retp(M,N,Ma,Na,Fv,J0,J0a);

endp;



/*  OLD STUFF
local Vb,Vs_1,Vdiff;
Vs_1   = 0;
Vb     = 0;
Vdiff  = 100;
do while Vdiff > 1E-6;    /*iterating on value matrix V*/
  Vb    = W + bet*M'*Vs_1*M;  /*J(0) = Sum[(bet^t)z(t)'Wz(t),t=0,...]*/
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;            /*Vs_1 is V(s-1), V is V(s)*/
endo;
"testing V and V-Vb";V;(V-Vb);


local Vb,Vs_1,Vdiff;
Vs_1   = 0;
Vb     = 0;
Vdiff  = 100;
do while Vdiff > 1E-6;
  Vb    = W + bet*Ma'*Vs_1*Ma;
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;
endo;
"testing V and V-Vb";V;(V-Vb);
*/
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**  DiscAlgR  Solves for robust optimization problem in discretionary case.
**
**            Same economy and loss function as in ComAlgR, but discretionary
**            equilibrium.
**
**
**
**  Usage:    {M,N,Ma,Na,Fu,Fv,J0,J0a}= DiscAlgR(A,B,Q,R,U,bet,n1,n2,Vt1,Ct1,
**                    ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,C,theta);
**
**
**  Input:    A        see ComAlgR
**            B
**            Q
**            R
**            U
**            bet
**            n1
**            n2
**            cutoff
**            Vt1      n1xn1 matrix. Initial guess for the value function
**                     (a positive definite matrix, such as the identity matrix)
**            Ct1      Initial guess for the matrix N. Ex: an n2xn1 zero matrix
**            ConvCrit scalar, convergence criterion (a small number, such as 1e-6)
**            Vweight  scalar, weight of value function in convergence test
**            Fweight  scalar, weight of policy function in convergence test
**            CritLags scalar, typically 1. If set to x, the convergence criterion is
**                     based on comparison of iteration t with iterations t-1,...,t-x
**            step     scalar in (0,1): factor of updating of V and F as
**                     in Vt1 = step*V + (1-step)*Vt1
**            PrintIt  scalar, 1: printing iteration number 2: printing iteration number
**                     and convergence criteria
**            MaxIter  scalar, maximum number of iterations (eg. 10000). NEW (March 2003)
**            C        see ComAlgR
**            theta
**
**  Output:   M        n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1) in the worst
**                     case solution, C1 is the first n1 rows of C
**            N        n2xn1 matrix, x2 = N*x1
**            Ma       n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1) in the approximating
**                     solution
**            Na       n2xn1 matrix, x2 = Na*x1, approximating model
**            Fu       kxn1 matrix, u = -Fu*x1
**            Fv       n1xn1 matrix, v = -Fv*x1
**            J0       scalar, loss function value, worst case
**            J0a      scalar, loss function, approximating model
**
**
**  Note:     The Na matrix (x2 = Na*x1) in the approximating model is the same as in the
**            worst case.
**
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
------------------------------------------------------------------------------*/

proc (8) = DiscAlgR( A,B,Q,R,U,bet,n1,n2,Vt1,Ct1,
                     ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,
                     C,theta );

local k,n,Btilde,Utilde,Rtilde,M,Nbig,V,F,A11,A12,A21,A22,B1,B2,C1,C2,Fu,Fv,Mw,NMw,
      Ma,Na,Ptilde,W,vecV,J0,J0a;


Q = (Q + Q')/2;             /*to make symmetric*/
R = (R + R')/2;

k = rows(R);
n = n1 + n2;

if not (C[n1+1:n,.] == 0);
  "Non-zero elements found in the last n2 rows of C";
  retp(error(0),error(0),error(0),error(0),error(0),error(0),error(0),error(0));
endif;


Btilde = B ~ C;
Utilde = U ~ zeros(n1+n2,n1);
Rtilde = (R           ~   zeros(k,n1)   )|
         (zeros(n1,k) ~  -theta*eye(n1) );



{M,Nbig,V,F} = DiscAlg( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,Vt1,Ct1,
                        ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter );

A11 = A[1:n1,1:n1];
A12 = A[1:n1,(n1+1):n];
A21 = A[(n1+1):n,1:n1];
A22 = A[(n1+1):n,(n1+1):n];

B1 = B[1:n1,.];
B2 = B[n1+1:n,.];

C1 = C[1:n1,.];
C2 = C[n1+1:n,.];

Fu = F[1:k,.];
Fv = F[k+1:k+n1,.];

Mw  = A11 + A12*Nbig - B1*Fu - C1*Fv;
NMw = A21 + A22*Nbig - B2*Fu;

"\lM-Mw";(M-Mw);
"N*M-NMw";(Nbig*M-NMw);


Ma  = A11 + A12*Nbig - B1*Fu;
/*Na  = NMw*inv(Ma);*/
Na  = Nbig;


Ptilde = eye(n1)| NA| -Fu| -Fv;              /*value function, worst case*/
W      = Ptilde' * ((Q~Utilde)|(Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n1*n1) - (M').*.(bet*M'));
V      = reshape(vecV,n1,n1)';                 /*solves V = W + bet*M'*V*M*/
if bet == 1;
  J0 = sumc( diag(C1'*V*C1*eye(n1)) );
else;
  J0 = bet/(1-bet) * sumc( diag(C1'*V*C1*eye(n1)) );
endif;


Ptilde = eye(n1)| Na| -Fu| zeros(n1,n1);              /*value function, approx*/
W      = Ptilde' * ((Q~Utilde)|(Utilde'~Rtilde)) * Ptilde;
vecV   = vec(W)/(eye(n1*n1) - (Ma').*.(bet*Ma'));
V      = reshape(vecV,n1,n1)';                 /*solves V = W + bet*Ma'*V*Ma*/
if bet == 1;
  J0a = sumc( diag(C1'*V*C1*eye(n1)) );
else;
  J0a = bet/(1-bet) * sumc( diag(C1'*V*C1*eye(n1)) );
endif;


Na = Nbig;

retp(M,Nbig,Ma,Na,Fu,Fv,J0,J0a);

endp;


/*  OLD STUFF
local Vb,Vs_1,Vdiff;
Vs_1  = 0;
Vb    = 0;
Vdiff = 100;
do while Vdiff > 1E-6;     /*iterating on value matrix V*/
  Vb    = W + bet*M'*Vs_1*M;   /*J(0) = Sum[(bet^t)z(t)'Wz(t),t=0,...]*/
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;             /*Vs_1 is V(s-1), V is V(s)*/
endo;
"testing V and V-Vb";V;(V-Vb);

local Vb,Vs_1,Vdiff;
Vs_1   = 0;
Vb     = 0;
Vdiff  = 100;
do while Vdiff > 1E-6;
  Vb    = W + bet*Ma'*Vs_1*Ma;
  Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
  Vs_1  = Vb;
endo;
"testing V and V-Vb";V;(V-Vb);
*/
/*----------------------------------------------------------------------------*/



/*------------------------------------------------------------------------------
**  SimpAlgR  Solving robust RE model for given simple rules of both policy maker
**            and evil agent.
**
**            Same economy and loss function as in ComAlgR, but
**            policy is now postulated to follow
**
**            u(t) = -Fu*x(t), where Fu is a given kxn matrix.
**
**
**  Usage:    dlibrary -a PsdGees;                @loading dll@
**            {M,N,Ma,Na,J0,J0a} = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,
**                                           x10,cutoff,C,theta,Fv,J0aIt );
**
**
**  Input:    A        see ComAlgR
**            B
**            Q
**            R
**            U
**            bet
**            n1
**            n2
**            Fu       kxn matrix, policy maker's decision rule u = -Fu*x
**            x10      n1x1, initial values of x1(t) vector
**            cutoff   same as ComAlgR
**            C        see ComAlgR
**            theta
**            Fv       n1xn matrix, evil agent's decision rule v = -Fv*x, last n2 columns
**                     should be zero
**            J0aIt    scalar, if 1: calculate loss for approx model
**
**  Output:   M        n1xn1 matrix, x1(t+1) = M*x1(t) + C1*e(t+1), worst case solution
**            N        n2xn1 matrix, x2 = N*x1, worst case
**            Ma       n1xn1 matrix, x1(t+1) = Ma*x1(t) + C1*e(t+1), approximating model
**            Na        n2xn1 matrix, x2 = Na*x1,  approximating model
**            J0       scalar, loss function value, worst case
**            J0a      scalar, loss function value, approximating case
**
**
**  Note:     The Na matrix (x2 = Na*x1) in the approximating model is the same as in the
**            worst case.
**
**
**
**
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
------------------------------------------------------------------------------*/

proc (6) = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,x10,cutoff,C,theta,Fv,J0aIt );

local k,n,Btilde,Utilde,Rtilde,C1,M,Nbig,J0,P,MM,Mw,NMw,Ma,Na,
      Ptilde,W,vecV,V,J0a;


Q = (Q + Q')/2;             /*to make symmetric*/
R = (R + R')/2;

k = rows(R);
n = n1 + n2;

if not (Fv[.,n1+1:n] == 0);
  "Non-zero elements found in the last n2 columns of Fv";
  retp(error(0),error(0),error(0),error(0),J0,error(0));
endif;

if not (C[n1+1:n,.] == 0);
  "Non-zero elements found in the last n2 rows of C";
  retp(error(0),error(0),error(0),error(0),error(0),error(0));
endif;


Btilde = B ~ C;
Utilde = U ~ zeros(n1+n2,n1);
Rtilde = ( R           ~ zeros(k,n1)    )|
         ( zeros(n1,k) ~ -theta*eye(n1) );

C1 = C[1:n1,.];


{M,Nbig,J0} = SimpRulT( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,Fu|Fv,C1*C1',x10,cutoff );
if ismiss(M);
  J0 = 0;          /*evil agent wants to maximize the loss: 0 is bad for him*/
  retp(error(0),error(0),error(0),error(0),J0,error(0));
endif;
/*M;*/

P   =  eye(n1)| Nbig;
MM  = (A - B*Fu - C*Fv)*P;
Mw  = MM[1:n1,.];
NMw = MM[n1+1:n,.];

/*"M-Mw";(M-Mw);
"N*M-NMw";(Nbig*M-NMw);*/


MM  = (A - B*Fu)*P;
Ma  = MM[1:n1,.];
Na  = Nbig;


if J0aIt == 1;            /*calculate loss for approx model*/
  Ptilde = ( eye(n)| -Fu| zeros(n1,n) ) * P;             /*value function, approx*/
  W      = Ptilde' * ((Q~Utilde)|(Utilde'~Rtilde)) * Ptilde;
  vecV   = vec(W)/(eye(n1*n1) - (Ma').*.(bet*Ma'));
  V      = reshape(vecV,n1,n1)';                 /*solves V = W + bet*Ma'*V*Ma*/
  if bet == 1;
      J0a = sumc( diag(C1'*V*C1*eye(n1)) );
  else;
      J0a = bet/(1-bet) * sumc( diag(C1'*V*C1*eye(n1)) );
  endif;
else;
  J0a = error(0);
endif;  /*if J0aIt*/


Na = Nbig;

retp(M,Nbig,Ma,Na,J0,J0a);

endp;


/*  OLD STUFF
local Vb,Vs_1,Vdiff;
  Vs_1  = 0;
  Vb    = 0;
  Vdiff = 100;
  do while Vdiff > 1E-6;
      Vb    = W + bet*Ma'*Vs_1*Ma;
      Vdiff = maxc(  maxc( abs(Vs_1-Vb) )  );
      Vs_1  = Vb;
  endo;
  "testing V and V-Vb";V;(V-Vb);
*/
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**  SimpAlgROpt   Solving robust RE model with simple policy rule,
**                optimizating evil agent's rule. Uses sqpSolve to
**                optimize SimpAlgRLoss wrt Fv.
**
**
**
**  Usage:    {M,N,Ma,Na,J0,J0a,FvX} = SimpAlgROpt( Fv0 );
**
**
**  Input:    Fv0     n1x(n1+n2) matrix, initial guesses of the evil agent's
**                    decision rule v = -Fv*x. The last n2 columns should be
**                    be zero.
**
**  Globals:  A, B, Q, R, U, bet, n1, n2, Fu, cutoff, C, theta: see SimpAlgR
**                       (note: globals are defined in the main program)
**
**  Output:   M        see SimpAlgR
**            N
**            Ma
**            Na
**            J0
**            J0a
**            FvX      n1xn matrix, optimial decision rule
**
**
**
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
------------------------------------------------------------------------------*/

declare matrix A       ?= .;            /*declaring the globals to avoid error*/
declare matrix B       ?= .;            /*messages if this file is executed before*/
declare matrix Q       ?= .;            /*these variables are set up by main program*/
declare matrix R       ?= .;
declare matrix U       ?= .;
declare matrix bet     ?= .;
declare matrix n1      ?= .;
declare matrix n2      ?= .;
declare matrix Fu      ?= .;
declare matrix cutoff  ?= .;
declare matrix C       ?= .;
declare matrix theta   ?= .;
declare matrix vv      ?= .;

proc (7) = SimpAlgROpt( Fv0 );

local par0,parX,fval,lagr,exitflag,FvX,M,N,Ma,Na,J0,J0a,k;


par0 = vec(Fv0[.,1:n1]);      /*sqpSolve wants a vector as argument*/

{parX,fval,lagr,exitflag} = sqpSolve(&SimpAlgRLoss,par0);        /*finding optimal Fv*/

FvX = reshape(parX,n1,n1)'  ~ zeros(n1,n2);          /*n1^2 vector -> n1xn matrix*/
"\largmax from sqpSolve is";FvX;

{M,N,Ma,Na,J0,J0a} = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,zeros(n1,1),cutoff,
                             C,theta,FvX,1 );   /*implications of optimal Fv*/

retp(M,N,Ma,Na,J0,J0a,FvX);

endp;
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**  SimpAlgRLoss    Loss function of evil agent in case with simple rule. Used
**                  in SimpAlgROpt
**
**
**  Usage:    J0 = SimpAlgRLoss( par );
**
**            see SimpAlgROpt
**
**
**
**
**
**
**
**
**  Paolo Giordani and Paul.Soderlind@unisg.ch, Jun 2001
------------------------------------------------------------------------------*/

proc (1) = SimpAlgRLoss( par );

local FvX,M,N,Ma,Na,J0,J0a,k;


FvX = reshape(par,n1,n1)'  ~ zeros(n1,n2);          /*n1^2 vector -> n1xn matrix*/

{M,N,Ma,Na,J0,J0a} = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,zeros(n1,1),cutoff,
                               C,theta,FvX,0 );
J0 = -J0;                   /*optimization routines are for minimizing*/

retp(J0);
endp;
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**  ErrDetProb    Simulate error detection probabilites for various sigma = -1/theta.
**                Model/equilibrium is chosen by a switch
**
**
**
**
**  Usage:    {rho,JcM} = ErrDetProb(A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C,
**                                   T,NSim,sigma,randnstate,ModelType,Fv0,MaxIter,constant);
**
**
**  Input:    A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C: see ComAlgR, ComAlgSR,
**                                                 DiscAlgR, or SimpAlgR
**            T            scalar, simulated sample length, Ex. 142
**            NSim         scalar, # of simulations,Ex 250
**            sigma        Kx1 vector, sigma = -1/theta values to calculate probs for,
**                         Ex. sigma = { -0.04, -0.03, -0.02, -0.01 };
**            randnstate   scalar, seed of random number generator, Ex. 123
**            ModelType    scalar, switch of model to simulate:
**                            1: ComAlgSR
**                            2: ComAlgR
**                            3: DiscAlgR
**                            4: SimpAlgR    (note: requires globals, see SimpAlgR)
**            Fv0          matrix, used only for simple rule, see SimpAlgROpt
**            MaxIter      scalar, maximum number of iterations (eg. 10000). NEW (March 2003)
**            constant     scalar, 1 if the first element of x is 1
**
**
**
**  Output:  rho          Kx1 vector, error detection probabilities
**           JcM          Kx2 matrix, constant in loss function, for
**                        worst case ~ approximating model
**
**
**
**
**
**
**  Paolo Giordani and Paul Sderlind, Paul.Soderlind@unisg.ch, Sep 2001
------------------------------------------------------------------------------*/

declare matrix theta ?= .;           /*global in proc for simple rule loss fn*/

proc (2) = ErrDetProb( A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C,
                       T,NSim,sigma,randnstate,ModelType,Fv0,MaxIter,constant);

local Kbig,rho,JcM,k,Mw,Nw,Ma,Na,Fv,J0,J0a,Mw2,xmeanw,Ma2,xmeana,rarwN,i,
      epsilon,Xw,Xa,x1p2,x2up1,x1,vw,va,ra,rw,paw;

Kbig = rows(sigma);

rho = miss(zeros(Kbig,1),0);            /*to put results in*/
JcM = miss(zeros(Kbig,2),0);

k = 1;
do until k > Kbig;                   /*loop over theta = -1/sigma*/


  if ModelType == 1;                 /*commitment, stable*/
    {Mw,Nw,Ma,Na,Fv,J0,J0a} = ComAlgSR( A,B,Q,R,U,bet,cutoff,C,-1/sigma[k] );
  elseif ModelType == 2;             /*commitment, unstable*/
    {Mw,Nw,Ma,Na,Fv,J0,J0a} = ComAlgR(  A,B,Q,R,U,bet,n1,n2,cutoff,C,-1/sigma[k] );
  elseif ModelType == 3;             /*discretion*/
    {Mw,Nw,Ma,Na,Fu,Fv,J0,J0a} = DiscAlgR( A,B,Q,R,U,bet,n1,n2,eye(n1),zeros(n2,n1),
                                           1e-6,0,1,0,1,0,MaxIter,C,-1/sigma[k] );
    Fv = Fv ~ zeros(n1,n2);          /*to make Fv n1xn as required below*/
  elseif ModelType == 4;
    theta = -1/sigma[k];             /*must be global in loss function proc*/
    {Mw,Nw,Ma,Na,J0,J0a,Fv} = SimpAlgROpt( Fv0 );
  endif;


  if constant == 1;
    Mw2    = Mw[2:rows(Mw),2:cols(Mw)];
    xmeanw = inv( eye(rows(Mw2))-Mw2 ) * Mw[2:rows(Mw),1];
    xmeanw = 1|xmeanw;
    Ma2    = Ma[2:rows(Ma),2:cols(Ma)];
    xmeana = inv( eye(rows(Ma2))-Ma2 ) * Ma[2:rows(Ma),1];
    xmeana = 1|xmeana;
  else;
    xmeanw = zeros(rows(Mw),1);
    xmeana = zeros(rows(Ma),1);
  endif;


  rndseed randnstate;


  rarwN = miss(zeros(NSim,2),0);
  i = 1;
  do until i > NSim;                             /*simulate NSim times*/
    epsilon = rndn(T+1,n1);                      /*iid N(0,I)*/
    if ModelType == 1;
      Xw = Var1SimPs(Mw,epsilon*C',T) + xmeanw';             /*simulate VAR(1)*/
      Xa = Var1SimPs(Ma,epsilon*C',T) + xmeana';
    elseif ModelType == 2;
      x1p2  = Var1SimPs(Mw,epsilon*C',T) + xmeanw';
      x2up1 = x1p2*Nw';
      Xw    = x1p2[.,1:n1] ~ x2up1[.,1:n2];
      x1p2  = Var1SimPs(Ma,epsilon*C',T) + xmeana';
      x2up1 = x1p2*Na';
      Xa    = x1p2[.,1:n1] ~ x2up1[.,1:n2];
    elseif ModelType == 3;
      x1 = Var1SimPs(Mw,epsilon*C[1:n1,.]',T) + xmeanw';
      Xw = x1 ~ x1*Nw';
      x1 = Var1SimPs(Ma,epsilon*C[1:n1,.]',T) + xmeana';
      Xa = x1 ~ x1*Nw';
    elseif ModelType == 4;
      x1 = Var1SimPs(Mw,epsilon*C[1:n1,.]',T) + xmeanw';
      Xw = x1 ~ x1*Nw';
      x1 = Var1SimPs(Ma,epsilon*C[1:n1,.]',T) + xmeana';
      Xa = x1 ~ x1*Nw';
    endif;
    vw = -Xw*Fv';
    va = -Xa*Fv';
    ra = sumc(sumc(va.*va))/2 - sumc(sumc(va.*epsilon[2:T+1,.]));  /*approximating model*/
    rw = sumc(sumc(vw.*vw))/2 + sumc(sumc(vw.*epsilon[2:T+1,.]));  /*worst case*/
    rarwN[i,.] = (ra<0) ~ (rw<0);
    i = i + 1;
  endo;

  paw      = meanc(rarwN);                          /*frequencies < 0*/
  rho[k]   = meanc(paw);                            /*(pa + pw)/2*/
  JcM[k,.] = J0 ~ J0a;                             /*loss fn [worst, approx]*/

  "\lsigma, theta, and rho ";
  (sigma[k] ~ -1/sigma[k] ~ rho[k] );


  k = k + 1;
endo;                     /*end loop over theta = -1/sigma*/


retp(rho,JcM);

endp;
/*----------------------------------------------------------------------------*/

