/*------------------------------------------------------------------------------
**  PsKalQui.prc
**
**
**
**
**
**
**
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch)
------------------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
**  KalQuickPs
**
**  Purpose:   Likelihood value of time invariant Kalman filter. The model is
**
**             y(t)    = Z*alfa(t) + d(t)+ e(t),             (measurement eq)
**             alfa(t) = TCap*alfa(t-1) + c(t) + R*eta(t)    (transition eq),
**
**             with
**
**             y(t)           Nx1 observed variables and
**             e(t)           Nx1 measurement errors in measurment eq.
**             alfa(t)        mx1 unobserved state variables,
**             eta(t)         gx1 disturbances in transition eq.
**
**             e(t) and eta(t) are assumed to be uncorrelated.
**
**
**
**  Usage:     LtMat = KalQuickPs( y,Z,d,TCap,c,R,H,Q,a0,P0,Diffuse,ToSS );
**
**  Input:     y     is a  TxN matrix of observed variables
**                         (which immediately will be transposed)
**             Z           Nxm "observation matrix"
**             d           TxN or (if constant) 1xN matrix of intercepts
**                         in measurement eq (will be transposed)
**             TCap        mxm "system matrix"
**             c           Txm of (if constant) 1xm matrix of intercepts
**                         in transition eq  (will be transposed)
**             R           mxg "input matrix"
**             H           NxN covariance matrix of e(t)
**             Q           gxg covariance matrix of eta(t)
**             a0          mx1 estimator of alfa(0) based on I(0)
**             P0          mxm covariance matrix of estimation error in a0,
**                         ie. E[(alfa(0)-a0)(alfa(0)-a0)'].
**             Diffuse     1 if diffuse prior i used: observations 1,...,m
**                         are replaced by missing values
**             ToSS        1 if limP(t)=P, which is tested, 0 otherwise
**
**  Output:    LtMat       Tx1 vector of log likelihood function values.
**
**
**
**
**  Reference: Harvey (1989), Forecasting, Structural Time Series Models.,,,,
**             pp. 105-
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), revised 21 September 1998
-----------------------------------------------------------------------*/

Proc  KalQuickPs( y,Z,d,TCap,c,R,H,Q,a0,P0,Diffuse,ToSS );

  local Tbig,m,N,aMat,at_1,Pt_1,t,att_1,Ptt_1,F,A,P,v,LtMat,Pmat,vv,Finv,SS,
        Ptt_1Mat,aTbigMat,at1Tbig,PTbigMat,Pt1Tbig,Pt1t,Pstar,atT,PtT,RQR;

  TBig = rows(Y);
  m = rows(TCAp);
  N = rows(H);

  y = y';     /*within proc: t runs along a row: y(t) is a column vector*/
  if rows(d) == 1;            /*Adjusting dimensions: scalar to vector, etc*/
    d = ones(1,TBig).*(d');
  else;
    d = d';
  endif;
  if rows(c) == 1;
    c = ones(1,TBig).*(c');
  else;
    c = c';
  endif;

  LtMat = miss(zeros(TBig,1),0);              /*Log likelihood*/

  at_1 = a0;                              /*Loop for optimal prediction*/
  Pt_1 = P0;

  RQR = R*Q*R';
  SS = 0;
  t = 1;
  do until t > TBig;
    att_1 = TCap*at_1 + c[.,t];                    /*predictions eq*/

    if SS == 0;                                   /*if not reached SS*/
      Ptt_1 = TCap*Pt_1*TCap' + RQR;
      F = Z*Ptt_1*Z' + H;                          /*Updating equations*/
      /*Finv = inv(F);*/
      Finv = invpd(F);
      P = Ptt_1 - Ptt_1'Z'Finv*Z*Ptt_1;     /*Prediction eq: transpose 931220*/
    endif;

    v = y[.,t]-Z*att_1 - d[.,t];                   /*pred error y - ytt-1*/
    a = att_1 + Ptt_1*Z'Finv*v;                   /*pred equation*/
    LtMat[t] =  - ln(detl)/2 - v'Finv*v/2;   /*log likelihood(t) + constant*/

    If (ToSS==1) AND ( maxc( abs(Pt_1-P) ) < 1E-6 );
      SS = t;                                 /*if convergent P*/
      ToSS = 100;
    endif;

    at_1 = a;                                 /*For next period*/
    Pt_1 = P;
    t = t + 1;
  endo;
  LtMat = -N/2*ln(2*Pi) + LtMat;        /*fixing constant in LogL*/


  if Diffuse == 1;
    LtMat[1:m] = miss(zeros(m,1),0);
  endif;

  retp( LtMat );

endp;
/*----------------------------------------------------------------------------*/

