/*--------------------------------------------------------------------------
**  PsSchur.prc
**
**
**
**
**
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch)
----------------------------------------------------------------------------*/



/*------------------------------------------------------------------------
**  Schur2Ps
**
**  Purpose:   Calculate the complex Schur decomposition and reorder so that
**             the roots come in ascending order.
**
**
**
**  Usage:     { T,Q } = Schur2Ps(a);
**
**  Input:     A     -    nxn matrix
**
**  Output:    T          nxn matrix, upper triangular, diagonal elements
**                        sorted in acending order (in absolute values)
**             Q          nxn matrix, unitary (orthogonal) such that A=Q*T*Q'
**
**
**
**  Remark 1:  The decomposition is such that
**
**             Q'Q = Q*Q' = I
**
**             A = Q*T*Q'  and T = Q'A*Q
**
**             T is upper triangular
**
**             The diagonal elements of T are ordered in ascending absolute
**             value
**
**
**  Remark 2 : This is a modification of code written by Paul Klein and
**             Mark Watson. Any errors are probably due to me.
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 1997
------------------------------------------------------------------------*/

proc (2) = Schur2Ps(a);

  local T,Q;

  { T,Q } = Reorder3Ps( schtoc(schur(a)) );  /*reorder the complex Schur form*/

  retp(T,Q);

endp;
/*------------------------------------------------------------------------*/



/*------------------------------------------------------------------------
**  Reorder3Ps
**
**  Purpose:   Reorders the eigenvalues of a Schur decomposition.
**
**
**  Usage:     { T,Q } = Reorder3Ps( T,Q );
**
**  Input:     T          nxn matrix, upper triangular,
**             Q          nxn matrix, unitary (orthogonal) such that A=Q*T*Q'
**
**  Output:    T          nxn matrix, upper triangular, diagonal elements
**                        sorted in acending order (in absolute values)
**             Q          nxn matrix, unitary (orthogonal) such that A=Q*T*Q'
**
**
**
**  Remark 1:  The value of Q*T*Q' is unchanged by this procedure.
**
**  Remark 2:  This algorithm cannot be used on the real Schur form
**             if there are 2x2 bumps (complex eigenvalues).
**             Should work on the complex Schur form.
**
**  Remark 3:  This is a modification of code written by Paul Klein and
**             Mark Watson. Any errors are probably due to me.
**
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 1997
------------------------------------------------------------------------*/

proc (2) = Reorder3Ps( t,q );

  local n,k,giv;

  n = rows(t);

  do until sortc( abs(diag(t)),1 ) == abs(diag(t));  /*sort: ascending order*/
    k = 1;
    do while k < n;                       /*loop over all diagonal elements*/
      if abs(t[k,k]) > abs(t[k+1,k+1]);   /*if ..., then change places*/
        giv = GivensPs( t[k,k+1]|t[k+1,k+1]-t[k,k] );
        t[k:k+1,k:n] = giv't[k:k+1,k:n];
        t[1:k+1,k:k+1] = t[1:k+1,k:k+1]*giv;
        q[1:n,k:k+1] = q[1:n,k:k+1]*giv;
      endif;
      k = k + 1;
    endo;
  endo;

  retp( t,q );

endp;
/*------------------------------------------------------------------------*/



/*-----------------------------------------------------------------------
**  GivensPs
**
**  Purpose:   Creates a Givens rotation matrix.
**
**
**  Usage:     g = GivensPs(x);
**
**  Input:     x    - 2x1 vector of complex numbers
**
**  Output:    g      2x2 unitary matrix such that the second element of g'x
**                    is zero.
**
**
**  Remark:    Based on code by Paul Klein. Any errors are probably due
**             to me.
**
**
**  Reference: Golub and van Loan (1996), Matrix Computations,3rd ed., 215-216
**
**
**  Paul Soderlind (Paul.Soderlind@unisg.ch), 1997
-----------------------------------------------------------------------*/

proc GivensPs(x);

  local a,b,tau,c,s,utmat;

  a = x[1];
  b = x[2];

  if b == 0;
    c = 1;
    s = 0;
  else;
    if abs(b) > abs(a);
      tau = -a/b;
      s = 1/sqrt(1+tau'tau);
      c = s'tau;
    else;
      tau = -b/a;
      c = 1/sqrt(1+tau'tau);
      s = c'tau';
    endif;
  endif;

  utmat = (   c   ~  s   )|
          ( (-s') ~ (c') );       /*to accomodate complex numbers*/

  retp( utmat );

endp;
/*-----------------------------------------------------------------------*/

