/*-----------------------------------------------------------------------------
%
% Program: test1.prg
%
%  Solves Svensson's 1997 model with robustness.
%  The model is given by the two equations
%
%  x(t+1)  = b1x(t)-b2(i(t)-pi(t))+u(t+1)
%  pi(t+1) = pi(t)+a1x(t)+e(t+1)
%
%  where x is the output gap, i the nominal interest rate, pi the inflaiton rate,
%  u an AD shock and e a cost push shock.
%
%  The central banker sets i. The loss function for the central banker is
%  quadratic in x , pi, i and i(t)-i(t-1).
%
%  Paolo Giordani and Paul Sderlind, February 2002
%-----------------------------------------------------------------------------*/

theta = 10;        /*degree of robustness. 5-100*/
AD = 1;            /*1 for AD shock, else cost-push shock*/

Tbig  = 10;        /*Periods to calculate impulse response for*/

/*coefficients of the law of motion*/

a1 = 0.4;          /*Ball (1997) uses a1=0.4 b1=0.8 b2=1 coeffinfl=1*/
b1 = 0.8;
b2 = 1;
coeffinfl = 1;    /*the coefficient on lagged inflation in the Phillips curve*/

/*coefficients for matrix C*/

stdinfl = 0.5;     /*std of the cost-push shock*/
stdy = 0.7;        /*of AD shock*/
stdi = 0.7;        /*of the ad hoc monetary policy shock*/

/*coefificents in loss function*/

bet = 0.98;          /*discount factor*/
outputweight = 0.5;  /*weight of squared output gap in the loss function*/
drweight = 0.2;      /*coefficient on (i(t)-i(t-1))^2 in loss function*/
rweight = 0;         /*coefficient on i(t)^2 in loss function  */

cutoff = 1.01;
/*----------------------------------------------------------------------------*/

library pgraph;                   /*activate graphics library*/

dlibrary PsDtgSen;           /*making dll code available: commitment case*/

#include monpol31.prc;            /*a simple model*/
#include psvarsim.prc;            /*calculate IRF*/
#include PsTgSen.prc;             /*generalized Schur decomposition*/
#include PsGees.prc;              /*Schur decomposition*/
#include MonEEA.prc;              /*solution algorithms*/
#include MonRob.prc;
/*----------------------------------------------------------------------------*/

/*writing the state-space form*/
{A,B,Q,U,R,uy,upi,C} = Svensson(a1,b1,b2,coeffinfl,stdinfl,stdy,outputweight,rweight,drweight);

if AD == 1;
  shock=C*uy;
else;
  shock=C*upi;
endif;
/*----------------------------------------------------------------------------*/

n1=3; n2=0;  /*no forward-looking variable*/

{M,N,Ma,Na,Fv,J0,J0a} = ComAlgSR( A,B,Q,R,U,bet,cutoff,C,theta );

x1w       = Var1SimPs(M,shock,Tbig);
interestw = x1w*N';
interestw = interestw[.,1];
Simupi_Dw = x1w[.,1:2]~interestw;   /*IRF of pi(t),y(t),i(t) to shock, worst case*/
/*----------------------------------------------------------------------------*/

x1a       = Var1SimPs(Ma,shock,Tbig);         /* approximating model  */
interest  = x1a*N';
interest  = interest[.,1];
Simupi_Da = x1a[.,1:2]~interest;             /* IRF of pi(t),y(t),i(t) to price shock  */


/*Plotting graph*/

xx = seqa(1,1,Tbig);
if AD == 1;
  aux = "AD shock";
else;
  aux = "CP shock";
endif;

graphset;
begwind;
window(2,2,0);
  _plegstr = "approximating"\"\000worst case";
  _plegctl = 1;
  title("Output gap to " $+ aux);
  xy(xx,Simupi_Da[.,1]~Simupi_Dw[.,1]);
  nextwind;
  title("Inflation to " $+ aux);
  xy(xx,Simupi_Da[.,2]~Simupi_Dw[.,2]);
  nextwind;
  title("Short rate to " $+ aux);
  xy(xx,Simupi_Da[.,3]~Simupi_Dw[.,3]);
endwind;
/*----------------------------------------------------------------------------*/

dlibrary -d;                         /*unloading dll*/
end;



/*-----------------------------------------------------------------------------
*
* proc Svensson
*
*  Prepares the matrices needed in solving the svensson model.
*
*  Usage: {A,B,Q,U,R,uy,upi,C} = SvenssonSS(a1,b1,b2,coeffinfl,stdinfl,stdy,
*                                           outputweight,rweight,drweight);
*
*  Input:  a1, b1,b2, coeffinfl are defined as in the program puzzle\svensson
*          stdinfl and stdy are the standard deviations  of the cost push and AD shock
*          outputweight, rweight and drweight are the coefficient on output,
*          i (nominal interest rate) and di in the loss function
*          (coeff on infl normalized to one)
*
*  Output: as in MonPol31. C*C' is the VCV matrix
*
*Paolo Giordani, January 2002
------------------------------------------------------------------------------*/

proc(8) = Svensson(a1,b1,b2,coeffinfl,stdinfl,stdy,outputweight,rweight,drweight);

local A,B,Q,U,R,n1,n2,upi,uy,C;

n1 = 3;   /*state variables: yg(t), pi(t), i(t-1), allows interest rate smoothing*/
n2 = 0;
                 /*State space representation for (in order) output and inflation*/
A = eye(n1+n2);
A[1,1] = b1; A[1,2] = b2;
A[2,1] = a1;
A[3,3] = 0;

B = -b2|0|1;

upi = 0|1|0;          /*price shock*/
uy  = 1|0|0;          /*positive demand shock*/

Q = (outputweight~0~0)|(0~1~0)|(0~0~drweight);
R = rweight+drweight;
U = zeros(n1+n2,1);
U[3,1] = -2*drweight;
C = zeros(n1+n2,n1);
C[1,1] = stdinfl;
C[2,2] = stdy;

retp(A,B,Q,U,R,uy,upi,C);

endp;
/*----------------------------------------------------------------------------*/
